@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2010 Open Source Applications Foundation.
+ * Copyright (c) 2004-2011 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -137,8 +137,7 @@ PyObject *make_descriptor(PyObject *(*get)(PyObject *));
 
 #define DECLARE_TYPE(name, t_name, base, icuClass, init, dealloc)           \
 PyTypeObject name##Type = {                                                 \
-    PyObject_HEAD_INIT(NULL)                                                \
-    /* ob_size            */   0,                                           \
+    PyVarObject_HEAD_INIT(NULL, 0)                                          \
     /* tp_name            */   "icu."#name,                                 \
     /* tp_basicsize       */   sizeof(t_name),                              \
     /* tp_itemsize        */   0,                                           \
@@ -206,8 +205,7 @@ static PyObject *t_name##_new(PyTypeObject *type,                       \
     return (PyObject *) self;                                           \
 }                                                                       \
 PyTypeObject name##Type = {                                                 \
-    PyObject_HEAD_INIT(NULL)                                                \
-    /* ob_size            */   0,                                           \
+    PyVarObject_HEAD_INIT(NULL, 0)                                          \
     /* tp_name            */   "icu."#name,                                 \
     /* tp_basicsize       */   sizeof(t_name),                              \
     /* tp_itemsize        */   0,                                           \
@@ -264,8 +262,7 @@ PyObject *wrap_##name(icuStruct *object, int flags)                     \
 
 #define DECLARE_CONSTANTS_TYPE(name)                                    \
 PyTypeObject name##Type = {                                             \
-    PyObject_HEAD_INIT(NULL)                                            \
-    /* ob_size            */   0,                                       \
+    PyVarObject_HEAD_INIT(NULL, 0)                                      \
     /* tp_name            */   "icu."#name,                             \
     /* tp_basicsize       */   0,                                       \
     /* tp_itemsize        */   0,                                       \
@@ -319,7 +316,7 @@ PyTypeObject name##Type = {                                             \
         Py_INCREF(&className##Type);                                    \
         PyModule_AddObject(module, #className,                          \
                            (PyObject *) &className##Type);              \
-        registerType(&className##Type, typeid(className).name());	\
+        registerType(&className##Type, typeid(className).name());       \
     }
 
 #define REGISTER_TYPE(className, module)                                \
@@ -380,7 +377,7 @@ PyTypeObject name##Type = {                                             \
     }
 
 
-#define DECLARE_RICHCMP(name, t_name) \
+#define DEFINE_RICHCMP(name, t_name) \
     static PyObject *t_name ## _richcmp(t_name *self,                   \
                                         PyObject *arg, int op)          \
     {                                                                   \
@@ -407,4 +404,25 @@ PyTypeObject name##Type = {                                             \
     }
 
 
+#define DEFINE_ABSTRACT(t_name, name, method)                           \
+    static PyObject *t_name##_##method(t_name *self, PyObject *arg)     \
+    {									\
+        return PyErr_Format(PyExc_NotImplementedError,                  \
+			    "%s.%s() is abstract", #name, #method);	\
+    }
+
+
+#if U_ICU_VERSION_HEX < 0x04060000
+#define ISINSTANCE(obj, type)                                   \
+    ((obj)->getDynamicClassID() == type::getStaticClassID())
+#else
+#define ISINSTANCE(obj, type)                                   \
+    (dynamic_cast<type *>(obj) != NULL)
+#endif
+
+#define RETURN_WRAPPED_IF_ISINSTANCE(obj, type)                 \
+    if (ISINSTANCE(obj, type))                                  \
+        return wrap_##type((type *)(obj), T_OWNED)
+
+
 #endif /* _macros_h */
